// -*- mode: c++ -*-
#include "Python.h"
#include "structmember.h"

#include <CGAL/version_macros.h>
#if CGAL_VERSION_MAJOR >= 4 && CGAL_VERSION_MINOR >= 11
#define CGAL_NEWER_API_4_11
#endif

#define NPY_NO_DEPRECATED_API NPY_1_7_API_VERSION
#include <numpy/arrayobject.h>

#pragma GCC diagnostic ignored "-Wwrite-strings"

#ifndef SIZEOF_PY_HASH_T
typedef Py_ssize_t Py_hash_t;
#endif

// Some common functions for alpha_shape_3 and alpha_shape_2
template<class T> PyObject* cast_PyObj(T* obj) {
  return reinterpret_cast<PyObject*>(obj);
}

template<typename T>
T* GETPTR1D(PyArrayObject* ary, npy_intp i) {
  return reinterpret_cast<T*>(PyArray_GETPTR1(ary, i));
}

template<typename T>
T* GETPTR2D(PyArrayObject* ary, npy_intp i, npy_intp j) {
  return reinterpret_cast<T*>(PyArray_GETPTR2(ary, i, j));
}

static bool ArrayIsDoubleType(PyArrayObject* points) {
  if (PyArray_TYPE(points) == NPY_DOUBLE)
    return true;

  PyErr_SetString(PyExc_TypeError, "Array must be double");
  return false;
}

static bool ArrayHasCorrectShape(PyArrayObject* points, int dim, int weighted,
                                 int rel_homology) {
  npy_intp* dims = PyArray_DIMS(points);
  int correct_2nd_dim = dim + weighted + rel_homology;

  if ((PyArray_NDIM(points) == 2) && (dims[1] == correct_2nd_dim))
    return true;
  
  PyErr_SetString(PyExc_ValueError, "Wrong shape of array");
  return false;
}

template<typename Types, typename Element, typename Iterator, typename Creator>
PyObject* AlphaShape_tolist(
  typename Types::AlphaShape* self,
  Iterator begin, Iterator end, int num_elements, Creator create) {
  
  PyObject* list = PyList_New(num_elements);
  if (!list) return NULL;
  
  int index = 0;
  for (Iterator iter = begin; iter != end; ++iter, ++index) {
    Element* element = create(self, iter);
    if (!element) goto error;
    if (PyList_SetItem(list, index, cast_PyObj(element)) < 0)
      goto error;
  }
  
  return list;

error:
  Py_DECREF(list);
  return NULL;
}
